/** 
 *   Site testing for SKA candidate.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   The GPIB implementation for spectral analyzer is initially writen
 *   by Cecilia Morras (on LavView).
 * 
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO:
 *        !) Comunicacion con el analizador espectral
 *        *) llaves de RF via paralelo:
 *           - falta descripcion de hardware.
 *           - falta implementacion
 *        *) comunicacion con el rotador
 *           ! falta descripcion de hardware.
 *           - falta implementacion
 *        *)
 *        *)
 *
 *
 **/

#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <getopt.h>
#include <syslog.h>
#include <sys/types.h>
#include <unistd.h>

#include "my_stdio.h"
#include "ska_survey.h"
#include "observ_table.h"
#include "observation.h"
#include "analyzer.h"

/* Flags: */
//int verbose_flag      = 1; /* 0 */
int daemon_flag       = 0; /* 1 */
int log_flag          = 0; /* 1 */
int character_flag    = 1; /* 0 */

char *help_text[] ={
#       ifdef VERBOSE_MODE
	"  -v\t\t\t- verbose mode (incremental).",
#       endif
	"  -n, --nodaemon\t- no run as a daemon.",
	"  -l,[--log]\t\t- write events to syslog.",
#       ifdef DEBUG_MODE
	"  -c,[--character]\t- print in stderr all characters arrived from serial\n\t\t\t  line.",
#       endif
	"  -d\t\t\t- set data directoy (def.: "DATA_DIR")",
	"  -f\t\t\t- set observation file (def.: "OBSERV_FILE")",
	"  --help\t\t- this help !!!.",
	""
};


void usage(char *program_name)
{
	fprintf(stderr,
#               ifdef VERBOSE_MODE
		"Usage: \t%s [-vnlhdf][--nodaemon][--log][--help]\n",
#                else
		"Usage: \t%s [-nlhdf][--nodaemon][--log][--help]\n",
#               endif
		program_name);
}


void help()
{
	int i=0;

	while((char) *help_text[i] != 0)
		fprintf(stderr,"%s\n",help_text[i++]);
	fprintf(stderr,"\n");
}


int main(int argc, char **argv)
{
	int an_dev;
	int i;
	char /**name=NULL,*/ *data_dir=DATA_DIR;
	char *observ_file = OBSERV_FILE;
	char *program_name;
	int pid;
	struct observ *obs, *obsp;
	char *cp;
	int op;
	static struct option  const  longopts[] =
	{
		{"help", 0, 0, 101},
		{"nodaemon", 0, 0, 'n'},
		{"log", 0, 0, 'l'},
		{0,  0,  0,  0}
	};
  
	/* who is my name? */
	if ((cp = strrchr(argv[0], '/')) != NULL) program_name  = cp + 1;
	else program_name  = argv[0];
  
	verbose_flag = 0;
	opterr =  0;
	while  ((op = getopt_long(argc, argv, "vnlcd:f:",longopts, 0)) != EOF)
		switch( op ){
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
#               ifdef DEBUG_MODE
		case 'c':
			character_flag = 1;
			break;
#               endif
		case 101:
			usage(program_name);
			help();
			exit(-1);
		case 'n':
			daemon_flag = 0;
			break;
		case 'l':
			log_flag = 1;
			break;
		case 'd':
			data_dir = optarg;
			break;
		case 'f':
			observ_file = optarg;
			break;
		default:
			usage(program_name);
			exit(-1);
		}
	
	if(daemon_flag){
		/* Ok, a daemond */
		verbose_flag = 0;
		if ( (pid=fork()) != 0 ) {
			exit(0);
		}
	}
	
	if(log_flag){
		openlog(program_name, LOG_PID, LOG_DAEMON );
		SYSLOG(LOG_NOTICE, "Startup succeeded.");
	}

	PVERB(PINFO,"Reading observation file: %s\n", observ_file);
	obs = obsp = get_observ_table(observ_file);
	if( obs == NULL){
		SYSLOG(LOG_ERR, "Error opening observation file: %s.",
		       observ_file );
		exit(1);
	}

/**********/
	PVERB(PINFO,"#M StartFre StopFreq RBW  Swee R S\n");
	while( obsp->next != NULL){
		PVERB(PINFO,"%d %8.5f %8.5f %4d %4d %2d %c\n", 
		       obsp->mode, obsp->start_freq, obsp->stop_freq,
		       obsp->RBW, obsp->sweep, obsp->repetitions, 
		      obsp->switchs);
		obsp = obsp->next;
	}
/**********/

	an_dev = -1;
	i = 0;
	do {
	  an_dev = open_analyzer(AN_PADDR);
	  if (an_dev < 0) {
	    sleep(1);
	  }
	  i++;
	}while(an_dev < 0 && i < AN_RETRY_TIME_OUT);
	
	if (an_dev >= 0) {
		observ_mode( 1, an_dev, obs);
	}
	free_observ_table(obs);

	return 0;
}
